﻿# Copyright (C) 2009-2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of windows-coleta
#
# windows-coleta is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

#coding: utf-8
import os
import _winreg
import ctypes
import wmi
import socket
import struct
import glob
from xml.etree.ElementTree import Element, SubElement, Comment, tostring
from datetime import timedelta, date, datetime	

DATASIDPATH = os.environ["ProgramFiles"] + "\\DataSID"
PREVIOUSPATH = DATASIDPATH + "\\data\\previous"

# Return the value from key/subkey in windows registry
def getRegistryValue(key, subkey, value):
    key = getattr(_winreg, key)
    handle = _winreg.OpenKey(key, subkey)
    (value, type) = _winreg.QueryValueEx(handle, value)
    return value

def getPointInfo(info):
    ptInfoFile = open(DATASIDPATH + "\\conf\\pointInfo", "r")
    ptInfo = value = " "
    while (ptInfo != info and ptInfo != ""):
        try:
            (ptInfo, value) = ptInfoFile.readline().split("=")
        except:
            ptInfo = value = ""
    ptInfoFile.close()
    return value

# Return Windows version
def getOSInfo(key):
    return getRegistryValue("HKEY_LOCAL_MACHINE", "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", key)

# Return CPU model installed in the machine
def getCpu():
    return getRegistryValue("HKEY_LOCAL_MACHINE", "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "ProcessorNameString")

# Return the machine mac address
def getMac(host):
    # Check for api availability
    try:
        SendARP = ctypes.windll.Iphlpapi.SendARP
    except:
        raise NotImplementedError('Usage only on Windows 2000 and above')

    # Doesn't work with loopbacks
    if host == '127.0.0.1' or host.lower() == 'localhost':
        host = socket.gethostname()

    try:
        inetaddr = ctypes.windll.wsock32.inet_addr(host)
        if inetaddr in (0, -1):
            raise Exception
    except:
        hostip = socket.gethostbyname(host)
        inetaddr = ctypes.windll.wsock32.inet_addr(hostip)

    buffer = ctypes.c_buffer(6)
    addlen = ctypes.c_ulong(ctypes.sizeof(buffer))

    if SendARP (inetaddr, 0, ctypes.byref(buffer),
            ctypes.byref(addlen)) != 0:
        raise WindowsError('Retrieval of mac address(%s) - failed' % host)

    # Convert binary data into a string
    macaddr = ''

    for intval in struct.unpack('BBBBBB', buffer):
        if intval > 15:
            replacestr = '0x'
        else:
            replacestr = 'x'
        macaddr = ''.join([macaddr, hex(intval).replace(replacestr, '' ), ":"],)
    return macaddr.upper()

# Return the total physical memory installed in the machine
def getRam():
    c = wmi.WMI()
    mem = 0
    for i in c.Win32_PhysicalMemory():
        mem = mem + int (i.Capacity)
    return mem / 1024

# Return disk model
def getHDDModel(disk):
    return disk.Model

# Return disk size in Gigabytes
def getHDDSize(disk):
    return int(disk.Size) / (1024 * 1024 * 1024)

# Return the total size of non Windows partitions
def nonWindowsPartitionSize(c):
    size = 0
    for partition in c.Win32_DiskPartition():
        if partition.Description != "Installable File System":
            if partition.Description != "Unknown":
                size = size + int (partition.Size)
    return size

# Return the total used disk
def diskUsed(c):
    free = 0
    size = 0
    for disk in c.Win32_LogicalDisk (DriveType = 3):
        free = free + int (disk.FreeSpace)
        size = size + int (disk.Size)
    return size - free

# Main collect function. This function gets the machine information and creates
# the XML
def collect():
    mainTag = Element('collected-data')

    # Collect point's info
    idpoint = getPointInfo("idponto")
    user_count = getPointInfo("usuarios")

    tagPointInfo = SubElement(mainTag, 'point-info')
    tagIdPoint = SubElement(tagPointInfo, 'idpoint')
    tagIdPoint.text = idpoint
    tagtUserCount = SubElement(tagPointInfo, 'user_count')
    tagtUserCount.text = user_count

    # Collect Version
    tagAgent = SubElement(mainTag, "agent-version")
    tagAgent.text = getRegistryValue("HKEY_LOCAL_MACHINE", "SOFTWARE\\DataSID", "Version")    

    # Inventory
    tagInventory = SubElement(mainTag, 'inventory')

    # Collect Hard Disk
    c = wmi.WMI()
    tagInventoryDisks = SubElement (tagInventory, 'disks')

    i = -1
    for disk in c.win32_DiskDrive():
        if disk.MediaType != "Removable Media":
            i = i + 1
            hddSize = getHDDSize(disk)
            hddModel = getHDDModel(disk)
            used = nonWindowsPartitionSize(c) + diskUsed(c)
            used = used / (1024 * 1024 * 1024)
            tagInventoryDisk = SubElement(tagInventoryDisks, 'disk')
            tagInventoryId = SubElement(tagInventoryDisk, 'id')
            tagInventoryId.text = str(i)
            tagInventoryDiskValue1 = SubElement(tagInventoryDisk, 'model')
            tagInventoryDiskValue1.text = hddModel
            tagInventoryDiskValue2 = SubElement(tagInventoryDisk, 'size')
            tagInventoryDiskValue2.text = str(hddSize)
            tagInventoryDiskValue3 = SubElement(tagInventoryDisk, 'used')
            tagInventoryDiskValue3.text = str(used)

    # Collect Memory
    memory = getRam()

    tagInventoryMemory = SubElement(tagInventory, 'memory')
    tagInventoryMemory.text = str(memory)     

    # Collect OS
    os = getOSInfo("ProductName")

    tagInventoryDistro = SubElement(tagInventory, 'distro')
    tagInventoryDistro.text = os
    tagInventoryKernel = SubElement(tagInventory, 'kernel')
    tagInventoryKernel.text = 'Windows'
    tagInventoryOs = SubElement(tagInventory, 'os')
    tagInventoryOs.text = 'Windows'

    # Collect CPU
    cpuVersion = getCpu()

    tagInventoryCPU = SubElement ( tagInventory, 'processor')
    tagInventoryCPU.text = cpuVersion

    # Collect Mac Address
    macAddress = getMac('localhost')
    macAddress = macAddress[0:-1]

    tagMac = SubElement(mainTag, 'interfaces')
    tagMacValue = SubElement(tagMac, 'interface', {'name':'', 'mac-address':macAddress})

    tagMachineType = SubElement(mainTag, 'machine-type')
    tagMachineType.text = 'client'

    tagMirrorsTimestamp = SubElement(mainTag, 'mirrors-timestamp')
    tagMirrorsTimestamp.text = " "

    tagUserHistory = SubElement(mainTag, 'user-history')
    tagUserHistory.text = " "

    return mainTag

# Main collect function. This function gets the net usage information and creates
# the XML    
def netCollect(log):
    mainTag = Element('net-collected-data')

    # Collect Version
    tagAgent = SubElement(mainTag, 'agent-version')
    tagAgent.text = getRegistryValue("HKEY_LOCAL_MACHINE", "SOFTWARE\\DataSID", "Version")

    # Collect Point's id
    idpoint = getPointInfo("idponto")
    tagPointId = SubElement(mainTag, 'id-point')
    tagPointId.text = idpoint

    # Collect MAC Address
    macAddress = getMac('localhost')
    macAddress = macAddress[0:-1]
    tagMac = SubElement(mainTag, 'interfaces')
    tagMacValue = SubElement(tagMac, 'interface', {'name':'', 'mac-address':macAddress})

    # Collect network usage
    tagUseBandUsage = SubElement(mainTag, 'bandwidth-usage')
    for t in glob.glob(PREVIOUSPATH + "\\*"):
        traffic = file(t, "r")
        trafficDate = traffic.readline().split("\n")[0]
        dateTmp = datetime.strptime(trafficDate, '%Y-%m-%d')
        intervalIdOld, rxPackageOld, rxBytesOld, txPackageOld, txBytesOld = traffic.readline().split("\n")[0].split(" ")
        if dateTmp >= datetime.now() - timedelta(days=15):
            for line in traffic:

                intervalId, rxPackageAux, rxBytesAux, txPackageAux, txBytesAux = line.split(" ")
                txBytesAux = txBytesAux.split("\n")[0]
                rxPackage = int(rxPackageAux) - int(rxPackageOld)
                rxBytes = int(rxBytesAux) - int(rxBytesOld)
                txPackage = int(txPackageAux) - int(txPackageOld)
                txBytes = int(txBytesAux) - int(txBytesOld)

                if rxBytes >= 0 and txBytes >= 0:
                    # Since we want the mean time between the five minutes intervals,
                    # the minute counter starts at 2 and gets 5 minutes increments.
                    # Also the "seconds counter" is fixed at 30. So we always get
                    # something like HH:2:30 or HH:7:30 (mean times).
                    #
                    # Example: if interval = 10:15~10:20  =>  mean = 10:17:30
                    intervalId = int(intervalId) + 1
                    trafficTime = timedelta(minutes=intervalId * 5 - 3) + timedelta(seconds=30)
                    tagNetUse = SubElement(tagUseBandUsage, 'netuse')
                    
                    tagNetUseId = SubElement(tagNetUse, 'id')
                    tagNetUseId.text = str(intervalId)
                    
                    tagDate = SubElement(tagNetUse, 'date')
                    tagDate.text = trafficDate
                    
                    tagTime = SubElement(tagNetUse, 'time')
                    tagTime.text = str(trafficTime)
                    
                    tagRx = SubElement(tagNetUse, 'rx')
                    tagRxPackets = SubElement(tagRx, 'rx-packets')
                    tagRxPackets.text = str(rxPackage)
                    tagRxBytes = SubElement(tagRx, 'rx-bytes')
                    tagRxBytes.text = str(rxBytes)
                    
                    tagTx = SubElement(tagNetUse, 'tx')
                    tagTxPackets = SubElement(tagTx, 'tx-packets')
                    tagTxPackets.text = str(txPackage)
                    tagTxBytes = SubElement(tagTx, 'tx-bytes')
                    tagTxBytes.text = str(txBytes)
                    
                intervalIdOld, rxPackageOld, rxBytesOld, txPackageOld, txBytesOld = intervalId, rxPackageAux, rxBytesAux, txPackageAux, txBytesAux
            traffic.close()
        else:
            traffic.close()
            os.remove(t)
    return mainTag
